#include <cstdio>
#include <cstdlib>
#include <string>
#include <vector>
#include <cstdio>
#include <cstring>
extern "C" {
    #include "utility.h"
}
#include "simplereader.h"





#define MAXSTRING          2048
#define GADGET_SKIP        ReadUInt(icfile,&blklen,SWAPBYTES);
#define SIZEOFGADGETHEADER 256
#define MZERO             (1e-10)
#define X                  0
#define Y                  1
#define Z                  2

char DATA[MAXSTRING];

unsigned int blklen;
bool          SWAPBYTES = false;
int          FORMAT;
bool          LGADGET = false;
bool        VERBOSE = false;


long         IDmin =  4294967296;
long         IDmax = -4294967296; //simply for debugging purposes



//using namespace std;




std::vector<Particle> GadgetSimpleReader::getParticle(std::string filename,
                                  int format,
                                  bool swap,
                                  bool lgadget,
                                  bool verbose){

    std::vector<Particle> particles;
    gadget_header header;
    
    SWAPBYTES = swap;
    FORMAT = format;
    LGADGET = lgadget;
    VERBOSE = verbose;
    
    FILE   *icfile;
    char    infile_name[MAXSTRING];
    strcpy(infile_name, filename.c_str());
    
    if((icfile = fopen(infile_name,"rb")) == NULL){
        fprintf(stderr, "File corrupted: %s\n", infile_name);
        exit(1);
    }
    
    readHeader(icfile, header);
    

    
    //particles.reserve(partsToRead);
    
    readData(icfile, header, particles);
    
    fclose(icfile);
    int sizeP = particles.size()/2;
    std::vector<Particle> parts;
    for (int i = 0 ; i < sizeP; i++) {
        parts.push_back(particles[sizeP+i]);
    }
    
    return parts;
    
    
}

void GadgetSimpleReader::readData(std::FILE* icfile,
                                  gadget_header & header,
                                  std::vector<Particle> & particles){
    
    int32_t no_part = 0;
    int i;
    int pID = 0;
    long           ldummy;
    int            idummy;
    for(int i = 0; i < N_TYPE; i++){
        no_part += header.npart[i];
    }
    particles.resize(no_part);
    
    /*================= read in GADGET particles =================*/
    if(FORMAT == 2)
    {
        GADGET_SKIP;
        
        fread(DATA,sizeof(char),4,icfile);
        DATA[4] = '\0';
        GADGET_SKIP;
        
        GADGET_SKIP;
        
        if(VERBOSE)
            fprintf(stderr,"reading %s",DATA);
    }
    else
    {
        if(VERBOSE)
            fprintf(stderr,"reading ");
    }
    
    GADGET_SKIP;
    
    if(VERBOSE)
        fprintf(stderr,"(%8.2g MB) ... ",blklen/1024./1024.);
    
    pID = 0;
    for(int i=0;i<no_part;i++)
    {
        
        /* read */
        ReadFloat(icfile,&(particles[pID].Pos[0]),SWAPBYTES);
        ReadFloat(icfile,&(particles[pID].Pos[1]),SWAPBYTES);
        ReadFloat(icfile,&(particles[pID].Pos[2]),SWAPBYTES);
        
        pID ++;
        
    }
    if(VERBOSE)
        fprintf(stderr,"Pos[X]=%12.6g Pos[Y]=%12.6g Pos[Z]=%12.6g ... ",
                particles[no_part-1].Pos[X],
                particles[no_part-1].Pos[Y],
                particles[no_part-1].Pos[Z]);
    
    GADGET_SKIP;
    
    if(VERBOSE)
        fprintf(stderr,"(%8.2g MB) done.\n",blklen/1024./1024.);
    /*================= read in GADGET particles =================*/
    
    
    
    /*================= read in GADGET velocities =================*/
    if(FORMAT == 2)
    {
        GADGET_SKIP;
        
        fread(DATA,sizeof(char),4,icfile);
        DATA[4] = '\0';
        GADGET_SKIP;
        
        GADGET_SKIP;
        
        if(VERBOSE)
            fprintf(stderr,"reading %s",DATA);
    }
    else
    {
        if(VERBOSE)
            fprintf(stderr,"reading ");
    }
    
    GADGET_SKIP;
    if(VERBOSE)
        fprintf(stderr,"(%8.2g MB) ... ",blklen/1024./1024.);
    
    pID = 0;
    for(i=0;i<no_part;i++)
    {
        /* read */
        ReadFloat(icfile,&(particles[pID].Vel[0]),SWAPBYTES);
        ReadFloat(icfile,&(particles[pID].Vel[1]),SWAPBYTES);
        ReadFloat(icfile,&(particles[pID].Vel[2]),SWAPBYTES);
        
        pID ++;
    }
    
    if(VERBOSE)
        fprintf(stderr,"Vel[X]=%12.6g Vel[Y]=%12.6g Vel[Z]=%12.6g ... ",
                particles[no_part-1].Vel[X],
                particles[no_part-1].Vel[Y],
                particles[no_part-1].Vel[Z]);
    
    GADGET_SKIP;
    if(VERBOSE)
        fprintf(stderr,"(%8.2g MB) done.\n",blklen/1024./1024.);
    /*================= read in GADGET velocities =================*/
    
    
    /*================= read in GADGET id's =================*/
    if(FORMAT == 2)
    {
        GADGET_SKIP;
        
        fread(DATA,sizeof(char),4,icfile);
        DATA[4] = '\0';
        GADGET_SKIP;
        
        GADGET_SKIP;
        
        if(VERBOSE)
            fprintf(stderr,"reading %s",DATA);
    }
    else
    {
        if(VERBOSE)
            fprintf(stderr,"reading ");
    }
    
    GADGET_SKIP;
    if(VERBOSE)
        fprintf(stderr,"(%8.2g MB) ... ",blklen/1024./1024.);
    
    pID = 0;
    for(i=0;i<no_part;i++)
    {
        /* get proper position in Part[] array */
        
        if(LGADGET)
        {
            ReadLong(icfile,&ldummy,SWAPBYTES);
            particles[pID].ID = ldummy;
            //printf("%ld\n", ldummy);
        }
        else
        {
            ReadInt(icfile,&idummy,SWAPBYTES);
            particles[pID].ID = (long) idummy;
            //printf("%d\n", idummy);
        }
        
        /* check the ID range of the "halo" particles */
        if(header.npart[0] <= i && i < header.npart[0]+header.npart[1])
        {
            if(particles[pID].ID > IDmax) IDmax = particles[pID].ID;
            if(particles[pID].ID < IDmin) IDmin = particles[pID].ID;
        }
        pID ++;
    }
    
    if(VERBOSE)
        fprintf(stderr,"ID=%12ld ...  ",particles[no_part-1].ID);
    
    GADGET_SKIP;
    if(VERBOSE)
        fprintf(stderr,"(%8.2g MB) done.\n",blklen/1024./1024.);
    /*================= read in GADGET id's =================*/
    
    
}

gadget_header GadgetSimpleReader::getHeader(std::string filename,
                                     int format,
                                     bool swap,
                                     bool lgadget,
                                     bool verbose){
    
    gadget_header header;
    
    SWAPBYTES = swap;
    FORMAT = format;
    LGADGET = lgadget;
    VERBOSE = verbose;
    
    FILE   *icfile;
    char    infile_name[MAXSTRING];
    strcpy(infile_name, filename.c_str());
    
    if((icfile = fopen(infile_name,"rb")) == NULL){
        fprintf(stderr, "File corrupted: %s\n", infile_name);
        exit(1);
    }
    
    readHeader(icfile, header);
    
    fclose(icfile);
    return header;
}


void GadgetSimpleReader::readHeader(FILE* icfile, gadget_header & header){
    char           DATA[MAXSTRING];
    
    /*================= read in GADGET IO header =================*/
    if(FORMAT == 2)
    {
        GADGET_SKIP;
        
        fread(DATA,sizeof(char),4,icfile);
        DATA[4] = '\0';
        GADGET_SKIP;
        
        GADGET_SKIP;
        
        if(VERBOSE)
            fprintf(stderr,"reading... %s\n",DATA);
    }
    
    GADGET_SKIP;
    
    ReadUInt(icfile,&(header.npart[0]),SWAPBYTES);
    ReadUInt(icfile,&(header.npart[1]),SWAPBYTES);
    ReadUInt(icfile,&(header.npart[2]),SWAPBYTES);
    ReadUInt(icfile,&(header.npart[3]),SWAPBYTES);    /* number of particles in current file */
    ReadUInt(icfile,&(header.npart[4]),SWAPBYTES);
    ReadUInt(icfile,&(header.npart[5]),SWAPBYTES);
    
    ReadDouble(icfile,&(header.mass[0]),SWAPBYTES);
    ReadDouble(icfile,&(header.mass[1]),SWAPBYTES);
    ReadDouble(icfile,&(header.mass[2]),SWAPBYTES);
    ReadDouble(icfile,&(header.mass[3]),SWAPBYTES);
    ReadDouble(icfile,&(header.mass[4]),SWAPBYTES);
    ReadDouble(icfile,&(header.mass[5]),SWAPBYTES);
    
    ReadDouble(icfile,&(header.time),SWAPBYTES);
    ReadDouble(icfile,&(header.redshift),SWAPBYTES);
    
    ReadInt(icfile,&(header.flag_sfr),SWAPBYTES);
    ReadInt(icfile,&(header.flag_feedback),SWAPBYTES);
    
    ReadInt(icfile,&(header.npartTotal[0]),SWAPBYTES);
    ReadInt(icfile,&(header.npartTotal[1]),SWAPBYTES);
    ReadInt(icfile,&(header.npartTotal[2]),SWAPBYTES);  /* total number of particles in simulation */
    ReadInt(icfile,&(header.npartTotal[3]),SWAPBYTES);
    ReadInt(icfile,&(header.npartTotal[4]),SWAPBYTES);
    ReadInt(icfile,&(header.npartTotal[5]),SWAPBYTES);
    
    ReadInt(icfile,&(header.flag_cooling),SWAPBYTES);
    ReadInt(icfile,&(header.num_files),SWAPBYTES);
    
    ReadDouble(icfile,&(header.BoxSize),SWAPBYTES);
    ReadDouble(icfile,&(header.Omega0),SWAPBYTES);
    ReadDouble(icfile,&(header.OmegaLambda),SWAPBYTES);
    ReadDouble(icfile,&(header.HubbleParam),SWAPBYTES);
    
    ReadInt(icfile,&(header.flag_stellarage),SWAPBYTES);
    ReadInt(icfile,&(header.flag_metals),SWAPBYTES);

    ReadInt(icfile,&(header.NallHW[0]),SWAPBYTES);
    ReadInt(icfile,&(header.NallHW[1]),SWAPBYTES);
    ReadInt(icfile,&(header.NallHW[2]),SWAPBYTES);  /* total number of particles in simulation */
    ReadInt(icfile,&(header.NallHW[3]),SWAPBYTES);
    ReadInt(icfile,&(header.NallHW[4]),SWAPBYTES);
    ReadInt(icfile,&(header.NallHW[5]),SWAPBYTES);
    
    ReadInt(icfile,&(header.flag_entropy_instead_u),SWAPBYTES);
    ReadInt(icfile,&(header.flag_doubleprecision),SWAPBYTES);
    ReadInt(icfile,&(header.flag_ic_info),SWAPBYTES);
    
    ReadFloat(icfile,&(header.lpt_scalingfactor),SWAPBYTES);
    
    
    ReadChars(icfile,&(header.fill[0]),
              256- N_TYPE*sizeof(uint32_t)- (6+N_TYPE)*sizeof(double)- (9+2*N_TYPE)*sizeof(int32_t)-sizeof(float));
    
    GADGET_SKIP;
   
    
    return;
}
