/***********************************************************
 *
 * Reader for gadget, use the default read_gadget.c
 *
 * Author: Lin F. Yang
 *
 * Date: Jan 24 2015
 *
 * *********************************************************/

#ifndef __LY_GADGET_READER__
#define __LY_GADGET_READER__

#include <string>
#include <cstdio>
#include <vector>
#include <stdint>
#include <iostream>
#include "utility.h"

/*=============================================================================
 *                              COMMON DEFINES
 *=============================================================================*/






#define MAXSTRING          2048 
#define GADGET_SKIP        ReadUInt(icfile,&blklen,SWAPBYTES);
#define SIZEOFGADGETHEADER 256
#define MZERO             (1e-10)
#define X                  0
#define Y                  1
#define Z                  2


/*=============================================================================
 *                                STRUCTURES
 *=============================================================================*/
class io_gadget_header
{
    int      np[6];
    double   massarr[6];
    double   expansion;
    double   redshift;
    int      flagsfr;
    int      flagfeedback;
    int      nall[6];
    int      flagcooling;
    int      NumFiles;
    double   BoxSize;
    double   Omega0;
    double   OmegaLambda;
    double   HubbleParam;
    char     unused[SIZEOFGADGETHEADER- 6*4- 6*8- 2*8- 2*4- 6*4- 2*4 - 4*8];
};

class info_gadget
{
  int      no_gadget_files;
  int      i_gadget_file;
  long   *(np[6]);
  long     nall;
    
  io_gadget_header header;
};

class particle_data 
{
  float     Pos[3];       /* particle position   */  
  float     Vel[3];       /* particle velocity   */  
  float     Mass;         /* particle mass       */
  float     u;            /* gas internal energy */
  long      ID;
};


/*=============================================================================
 *                                PROTOTYPES
 *=============================================================================*/

class DataReader{
public:
    /**
     *  format = 1: Gadget1, 2 gadget 2
     *  swap = true: perform byteswap
     *  lgadget = true: long ids
     *  *****/
    DataReader( int format = 1, 
            bool swap = true, 
            bool lgadget = true,
            bool verbose = false);
    
    ~DataReader();


    /**
     * set filename, 
     * return true if the file is good
     * return false if the file is bad
     */
    bool setFile(std::string dataFileName);
    
    
    io_gadget_header & getHeader();

    /**
     * Get a list of particles, from the startpart to the end
     **/
    std::vector<particle_data> getParticle(int64 startPart, int64 numparts);

    
    
private:
    info_gadget gadget_;

    std::string dataFileNames_;
    std::vector<particle_data> particles_;

    bool format_;
    bool swap_;
    bool lgadget_;
    bool filegood_;
    FILE *icfile_;
    void loadFile_();
    bool verbose_;
    unsigned int blklen_;
    
    long         IDmin =  4294967296;
    long         IDmax = -4294967296; //simply for debugging purposes
}



#endif
