//
//  Utils.hpp
//  PickDropSampling
//
//  Created by Alan Liu on 4/23/13.
//  Copyright (c) 2013 Alan. All rights reserved.
//

#ifndef PickDropSampling_Utils_hpp
#define PickDropSampling_Utils_hpp

#include <vector>
#define _USE_MATH_DEFINES
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <ctime>


#if defined (WINDOWS) || defined (_WIN32) || defined (__WIN32__)
#define OS_WINDOWS
#endif

#ifdef OS_WINDOWS
#include <windows.h>
typedef long long Clock;
#else
#include <sys/time.h>
#include <sys/resource.h>
typedef struct timeval Clock;
#endif


/**
 *@brief Start the clock
 *@param c pointer to Clock
 */
static inline void ClockStart(Clock * const c)
{
#ifdef OS_WINDOWS
    QueryPerformanceCounter((LARGE_INTEGER *) c);
#else
    gettimeofday(c, NULL);
#endif

}

static inline double ClockElapsed(Clock * const c)
{
#ifdef OS_WINDOWS
    long long end;
    long long freq;
    QueryPerformanceCounter((LARGE_INTEGER *) &end);
    QueryPerformanceFrequency((LARGE_INTEGER *) &freq);
    return ((double) (end - (*c))) / freq;
#else
    struct timeval end;
    gettimeofday(&end, NULL);
    return  (end.tv_sec  - c->tv_sec ) +  0.000001 * (end.tv_usec - c->tv_usec);
#endif
}
#endif
