#include <vector>
#include <cmath>
#include <cstdio>
#include <random>
#include <cstring>
#include <climits>
#include <algorithm>
#include <cstdlib>
#include <iostream>
#include "countSketch.h"

using namespace std;


random_device rd;
default_random_engine generator(rd());

/*
The HashFamily constructor
*/
void HashFamily::setHashFamily(long long int hashFamilySize, long long int range){
    familySize = hashFamilySize;
    n = range;
    srand (time(NULL));
    for (int i = 0; i < familySize; ++i) {
        aParams.push_back(rand() % primeNumber);
        bParams.push_back(rand() % primeNumber);
    }
}

/*
Hash functions
*/
int HashFamily::evalFunction(long long int x, long long int iFunction){
    return ((aParams[iFunction] * x + bParams[iFunction]) % primeNumber) % n;
}

CountSketch::CountSketch(double levelSize, int totalParts, double boxSize,
                         int b, int t, int k){
    levelSize_ = levelSize;
    totalParts_ = totalParts;
    boxSize_ = boxSize;

    b_=b;
    t_=t;
    k_=k;
    countSketch = new int[t_*b_];
    memset(countSketch,0,t_*b_);

    hashFamilyH.setHashFamily(t_,b_);
    hashFamilyS.setHashFamily(t_,2);

    topK.setK(k);
    
}


int CountSketch::getCell_(double x, double y, double z){
    int ix = (int)floor(x / levelSize_);
    int iy = (int)floor(y / levelSize_);
    int iz = (int)floor(z / levelSize_);

    int keysize = (int)ceil(boxSize_ / levelSize_);
    int key = ix + iy * keysize + iz * keysize * keysize;

    return key;
}


void CountSketch::add(int key){
    
    for (int i = 0; i < t_; i++) {
        int tmpB = hashFamilyH.evalFunction(key,i);
        int tmpS = hashFamilyS.evalFunction(key,i);

        if (tmpS == 1) {
            countSketch[i*b_+tmpB]+=1;
        }else
        {
            countSketch[i*b_+tmpB]-=1;
        }
    }
}

int CountSketch::estimate(int key){

//printf("########Estimate Oepration");
    int a[t_];
    for (int i=0; i<t_; i++) {
    int tmpB = hashFamilyH.evalFunction(key,i);
    int tmpS = hashFamilyS.evalFunction(key,i);
        if(tmpS == 0)
        {
            tmpS=-1;
        }
            a[i]=countSketch[i*b_+tmpB]*tmpS;
    }
    sort(a,a+t_);
    return a[t_/2];
    
}

void CountSketch::alg(double x, double y, double z){
    int key = getCell_(x, y, z);
    Cell cell(x,y,z, boxSize_,levelSize_);
    add(key);
    // vector<Cell>::iterator it = maxHeap.end();
    // vector<Cell>::iterator smallest = maxHeap.end();
    // int smallestCounter = numeric_limits<int>::max();
    
    // // unordered_map<long int, int>::const_iterator got = topK.find(key);


    // if(maxHeap.size()>0){
    //   for(vector<Cell>::iterator itTmp = maxHeap.begin();itTmp!=maxHeap.end();++itTmp){
    //     if((*itTmp).count<smallestCounter){
    //       smallestCounter = (*itTmp).count; 
    //       smallest = itTmp;
    //       }	
    //     if(getCell_((*itTmp).x, (*itTmp).y, (*itTmp).z)==key){	
    //       //printf("Really??? \n");
    //       it = itTmp;
    //     }
    //   }
    // }


    // if ( it!=maxHeap.end() && ! (topK.hasMember(key)) ){
    //           cout << "troubles1";
    //       }

    //  if (it==maxHeap.end() && topK.hasMember(key)){
    //           cout << "troubles2";
    //       } 
      
    //  // if (it!=maxHeap.end() && (*it).count != got->second){
    //  //      cout << "troubles3";
    //  // }

    //  //|| (*smallest).count != minCounter || getCell_((*smallest).x, (*smallest).y, (*smallest).z) != minCounterKey)
    //  if ((maxHeap.size() > 0)&&(smallestCounter!=topK.getMinCounter())){
    //       cout << "troubles4";
    //  }


    
    if(!(topK.isFull()) && !(topK.hasMember(key))){           
      cell.count = estimate(key);
      topK.insert(key, cell.count);
    }else if (topK.hasMember(key)) {
      topK.increment(key);
    }else{
      cell.count = estimate(key);
      if (topK.getMinCounter() < cell.count) {
        topK.insert(key, cell.count);
      }   
    }



    // if((int)(maxHeap.size())<k_ && it == maxHeap.end()){           
    //   //printf("insert into max Heap \n");
	   //  cell.count = estimate(key);
    //   //printf("Cell is %2.3f %2.3f %2.3f \n", cell.x,cell.y,cell.z);
    //   maxHeap.push_back(cell);
    // //  topK.insert(pair<long int, int> {key, cell.count});
    // //  if (cell.count < minCounter){
    // //    updateTopKMin();
    // //  }
    // } 
    // else if (it!=maxHeap.end()) {
    //   //printf("Increment the counter \n");
    //   cell.count = (*it).count +1;
    //   //printf("Size of maxHeap %d \n", cellCounter[key]);
    //   maxHeap.erase(it);
    //   maxHeap.push_back(cell);
    //   // topK[key] ++;
    //   // if (minCounterKey == key) {
    //   //   minCounter ++;
    //   //   updateTopKMin();
    //   // }
    // }
    // else{
    // 	if(estimate(key)>(*smallest).count){
	   //    //printf("Kick out the smallest \n");
    //     maxHeap.erase(smallest);
    //     //smallestC = make_pair(key,estimate(key));
    //     cell.count = estimate(key);
    //     maxHeap.push_back(cell);
    //   }
    //   // cell.count = estimate(key);
    //   // if (minCounter < cell.count) {
    //   //   topK.erase(minCounterKey);
    //   //   topK.insert(pair<long int, int> {key, cell.count});
    //   //   updateTopKMin();
    //   //   //topK.erase(getCell_((*smallest).x, (*smallest).y, (*smallest).z));
    //   //   //topK.insert(pair<long int, int> {key, cell.count});
    //   // }   
    // }   
}


Cell::Cell(double x_, double y_, double z_, 
            double boxSize_, double levelSize_){
      int ix = (int)floor(x_ / levelSize_);
      int iy = (int)floor(y_ / levelSize_);
      int iz = (int)floor(z_ / levelSize_);
       
      x = ix * levelSize_;
      y = iy * levelSize_;
      z = iz * levelSize_;
      //printf("Cell: %f %f %f\n", x_/levelSize_, y_/levelSize_,z_/levelSize_);
      count = 0;
}




vector<vector<int> > CountSketch::getResult(){
  vector<pair<long int,int>> counters = topK.getTopKcounters();
  long int keysize = (long int)ceil(boxSize_ / levelSize_);
  vector<vector<int> > output;
  for (int i = 0; i < (int) counters.size(); ++i)
  {
    long int key = counters[i].first;
    int ix = (key % (keysize*keysize)) % keysize;
    int iy = (key % (keysize*keysize)) / keysize;
    int iz = (key / (keysize*keysize));
    vector<int> center = {ix, iy, iz, counters[i].second};
    output.push_back(center);
  }
  return output;
}


CountSketch::~CountSketch(){
    delete[] countSketch;
}