#include <iostream>
#include <map>
#include <cstdio>
#include <cstring>
#include <vector>
#include <set>
#include <cmath>
#include <fstream>
#include "omp.h"

#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include "kd.h"
#include "hash.h"

//#include <gadgetreader.hpp>
#include "simplereader.h"
//#include "countSketch.h"
#include "pickdrop.h"
//#include "tipsydefs.h"


using namespace std;
//using namespace GadgetReader;

int getCell(double x, double y, double z, double boxSize_, double levelSize_){
    int ix = (int)floor(x / levelSize_);
    int iy = (int)floor(y / levelSize_);
    int iz = (int)floor(z / levelSize_);
    
    int keysize = (int)ceil(boxSize_ / levelSize_);
    int key = ix + iy * keysize + iz * keysize * keysize;
    
    return key;
}

vector<int> getPos(int key_, double boxSize_, double levelSize_){
    
    int keysize = (int)ceil(boxSize_ / levelSize_);
    int ix = (key_ % (keysize*keysize)) % keysize;
    int iy = (key_ % (keysize*keysize)) / keysize;
    int iz = (key_ / (keysize*keysize));
    
    return {ix, iy, iz};
}

struct compare {
    bool operator()(const vector<int> &a, const vector<int> &b) {
        if (a[3] > b[3])
            return true;
        else
            return false;
    }
};


int main(int argc, const char* argv[]){
    if(argc != 6){
        printf("Usage: %s <FileNamePattern> <nFiles> <levelSize> <nCenters> <particleSize>\n", argv[0]);
        exit(1);
    }
    
    double levelSize = atof(argv[3]); //the size of a cell
    int k = atoi(argv[4]);// number of centers to return

    srand(time(0));
    /*##########   parameters for Pick and Drop   #############*/
    int nInstances= 100*k; // # of pick and drop instances
    int tSize = 5; // # of columns in the matrix of each pick and drop instance
    double lambda = 1.0; //the parameter lambda
    /*##########   End of Parameters for Pick and Drop   ###########*/
    
    
    
    int partSize = atoi(argv[5]);
    long int totalParts = partSize*partSize*partSize;
    
    int nR = ceil(((double)totalParts)/((double)nInstances)/((double)tSize))+10;
    
    int nFiles = atoi(argv[2]);
    //double error = 0.05;
    //int b = 9999999;
    
    HashFamily hashFamilyH(1,nInstances);
    GadgetSimpleReader testreader;
    gadget_header head;
    
    string fileNamePattern = argv[1];
    head = testreader.getHeader(fileNamePattern + to_string(0), 1, true, false, true);
    
    double boxSize = head.BoxSize;
    cout<<"Boxsize is "<<boxSize<<endl;
    
    int keysize = (int)ceil(boxSize / levelSize);
    
    //double linkLength = 0.2*(sqrt(3*boxSize/(256*256*256)));
    
    long ncells = (boxSize*boxSize*boxSize)/(levelSize*levelSize*levelSize);
    
    //int t = log2(ncells/error);
    
    
    //double phi = (pow(ncells,1-1/2)*)/totalParts;
    
    //double delta =
    
    
    vector<vector<int> > shifts;
    shifts.push_back({0,0,0});
    shifts.push_back({0,0,1});
    shifts.push_back({0,1,0});
    shifts.push_back({0,1,1});
    shifts.push_back({1,0,0});
    shifts.push_back({1,0,1});
    shifts.push_back({1,1,0});
    shifts.push_back({1,1,1});
    
    //omp_set_num_threads ( 4 );
    
    vector<Particle> parts;//partices for a file
    
    //map<int, int> resPairs;
    //map<int, bool> resMap;
    
    vector<PickDrop> PDS;
    int PDSCounter =0;// 10k instances of pick and drop sampling
    
    
    for (int test =0; test < 41; test++) {
        
    ofstream output;
    char filename[23];
    sprintf(filename, "%d-PDOutput.txt",test);
    output.open(filename,fstream::out | ios_base::trunc);
    
    //#pragma omp parallel for
    for (int shift_i = 0; shift_i <8; ++shift_i)
    {
//        PDS = new PickDrop[nInstances];
//        printf("[OPENMP] The thread # %d is running \n",omp_get_thread_num());
        // 10k instances of pick and drop sampling
        for (int instanceI =0; instanceI<nInstances; instanceI++) {
            
            //PickDrop* tmp = new PickDrop(tSize,lambda,keysize,nR,rand());
            //printf("tmp adr %f \n", tmp);
            PDS.push_back(*new PickDrop(tSize,lambda,keysize,nR,rand()));
            if(instanceI % 1000 == 0){
                printf("Please wait... %2.2f%%         \r", (double)instanceI * 100 / nInstances);
                cout.flush();
            }
            
        }
        
        //CountSketch CS(levelSize, ncells, boxSize, b, t, k);
        
        vector<int> shift = shifts[shift_i];
        
        printf("Start Count Sketch # %d shift calculation...\n", shift_i);
        
        //vector<vector<int> >results = CS.getResult();
        
//        for(int kkk = 0; kkk<nInstances; kkk++)
//        {
//            
//            op<<PDS[kkk].counter<<endl;
//        }

        for (int iFile =0; iFile<nFiles; iFile++) {
        
        string filename = fileNamePattern + to_string(iFile);
        
        printf("Reading particles from data file #%d ... \n",iFile);
        parts = testreader.getParticle(filename, 1, true, false, true);
        int nparts = parts.size();
        //double boxSize = snap.GetHeader().BoxSize;
        //int t = log2(ncells/error);
        printf("Params: (levelsize %f) (nparts %d) (ncells %ld) (boxsize %f)\n",levelSize,nparts,ncells,boxSize);
            
            //vector<vector<int> >results = CS.getResult();
        
            
        for(int i = 0; i < nparts; i++){
            double x = parts[i].Pos[0] + shift[0]*0.5*levelSize;
            double y = parts[i].Pos[1] + shift[1]*0.5*levelSize;
            double z = parts[i].Pos[2] + shift[2]*0.5*levelSize;
            int key = getCell(x,y,z,boxSize,levelSize);
            
            int hashedKey = hashFamilyH.evalFunction(key,0);
            PDS[hashedKey].Add(key);
            
            if(i % 1000 == 0){
                printf("Please wait... %2.2f %\r", (double)i * 100 / nparts);
                cout.flush();
            }
        }
        printf("\n");
        
        
        //printf("Reading particles from the data file... \n");
        //vector<float> temppos = snap.GetBlock("POS ", nparts, 0, ~(1 << DM_TYPE));
        
    }

        for (int iPD = 0; iPD<nInstances; iPD++) {
            
            vector<int> tmpPD = PDS[iPD].PD();
            
            if (tmpPD[3]>100){
                output<<tmpPD[0]<<' '<<tmpPD[1]<<' '<<tmpPD[2]<<' '<<tmpPD[3]<<endl;
                PDSCounter++;
            }
        }
        
        
        vector<PickDrop>().swap(PDS);
}
    
    output.close();
    printf(" %d centers are stored.... \n", PDSCounter);
        
    }
    //vector<vector<int> >results = CS.getResult();
    
//    for(set<vector<int>,compare>::iterator it=resPairs.begin(); it!=resPairs.end(); it++)
//    {
//        
//        output<<(*it)[0]<<' '<<(*it)[1]<<' '<<(*it)[2]<<' '<<(*it)[3]<<endl;
//    }
    
//    for (int iFile =0; iFile<nFiles; iFile++) {
//        
//        string filename = fileNamePattern + to_string(iFile);
//        
//        printf("Second pass reading particles from data file #%d ... \n",iFile);
//        parts = testreader.getParticle(filename, 1, true, false, true);
//        int nparts = parts.size();
//        
//        
//        for(int iN = 0; iN < nparts; iN++){
//            double x = parts[iN].Pos[0];
//            double y = parts[iN].Pos[1];
//            double z = parts[iN].Pos[2];
//            int key = getCell(x,y,z,boxSize,levelSize);
//            if (resMap[key]==true) {
//                resPairs[key]+=1;
//            }
//        }
//
//    }
    
//    for (int ik=0; ik < resPairs.size(); ik++) {
//        if(resPairs[ik][3]>100)
//            output<<resPairs[ik][0]<<' '<<resPairs[ik][1]<<' '<<resPairs[ik][2]<<' '<<resPairs[ik][3]<<endl;
//    }
    
    
//    for (map<int,int>::iterator it=resPairs.begin(); it!=resPairs.end(); ++it)
//    {
//        vector<int> tmpPos = getPos(it->first,boxSize,levelSize);
//        output<<tmpPos[0]<<' '<<tmpPos[1]<<' '<<tmpPos[2]<<' '<<it->second<<endl;
//    }
    
    //snap.GetBlocks("POS ",data_array, snap.GetNpart(DM_TYPE), 0, 0);
    
    //map<int, Cell> sampledCells = sampler.getSampledCells();
    
    
    //printf("%d/%d cell opened. %2.2f%% of the cells are opened\n",
    
    //    for(it = sampledCells.begin(); it != sampledCells.end(); ++it){
    //        printf("%f %f %f %d\n",
    //                it->second.x, it->second.y, it->second.z, 
    //                it->second.count);
    //    }
    
    
 /*
  Combine the shift results
  */
    
    
    return 0;
}


