#include <iostream>
#include <cstdio>
#include <vector>
#include <cstring>
#include <cmath>
#include <fstream>
#include <gadgetreader.hpp>
#include <queue>          // std::priority_queue
#include <stdio.h>
#include <stdlib.h>


using namespace std;
using namespace GadgetReader;

class CompareDist
{
public:
    bool operator()(pair<int,int> n1,pair<int,int> n2)
    {
        if(n1.second > n2.second)
            return true;
        else
            return false;

    }
};




int main(int argc, const char* argv[]){
    if(argc != 3){
        printf("Usage: %s <filename> <levelSize>\n", argv[0]);
        exit(1);
    }

    double levelSize = atof(argv[2]);
    GSnap snap(argv[1]);
    int nparts = snap.GetNpart(DM_TYPE);
    double boxSize = snap.GetHeader().BoxSize;
    int cellsPerDimension = (int) boxSize/levelSize;
    long ncells = (cellsPerDimension+1)*(cellsPerDimension+1)*(cellsPerDimension+1);

    
    
    //printf("Params: (levelsize%f) (ncells%d) (boxsize%f) (b%d) (t%d)",levelSize,ncells,boxSize,b,t);
    printf("Reading particles from the data file... \n");
    vector<float> temppos = snap.GetBlock("POS ", nparts, 0, ~(1 << DM_TYPE));
    

    vector<vector<int> > shifts;
    shifts.push_back({0,0,0});
    shifts.push_back({0,0,1});
    shifts.push_back({0,1,0});
    shifts.push_back({0,1,1});
    shifts.push_back({1,0,0});
    shifts.push_back({1,0,1});
    shifts.push_back({1,1,0});
    shifts.push_back({1,1,1});
    
for (int shift_i = 0; shift_i < (int) shifts.size(); ++shift_i)
{
    vector<int> shift = shifts[shift_i];
    printf("Start Full Search ...\n");  
    vector<unsigned int> counters(ncells, 0);
    for(int i = 0; i < nparts; i++){
        double x = temppos[i * 3 + 0] + shift[0]*0.5*levelSize;
        double y = temppos[i * 3 + 1] + shift[1]*0.5*levelSize;
        double z = temppos[i * 3 + 2] + shift[2]*0.5*levelSize;
        long int key = floor(x/levelSize) + boxSize/levelSize* floor(y/levelSize) +  boxSize/levelSize*boxSize/levelSize*floor(z/levelSize);
        counters[key] ++;
        if(i % 1000 == 0){
            printf("Please wait... %2.2f%%         \r", (double)i * 100 / nparts);
            cout.flush();
        }
    }
    printf("\n");
    cout<<"Searching for top 1000..."<<endl;
    priority_queue<pair<int,int>,vector<pair<int,int>>,CompareDist> pq;
    for(int i = 0; i < (int)counters.size(); i++){
        if ((pq.size() < 1000)||((int)counters[i] > (int)pq.top().second)){
            pq.push(pair<int,int> (i, counters[i]));}
        if (pq.size() > 1000) {pq.pop();}
        if(i % 10000 == 0){
            printf("Please wait... %2.2f%%         \r", (double)i * 100 / counters.size());
            cout.flush();
        }
    }

    vector<int> keys;
    while(pq.size() > 0){
        keys.push_back(pq.top().first);
        pq.pop();
    }    

    vector<vector<int> > results;
    for (int i = 0; i < (int)keys.size(); ++i)
    {
        int x = (keys[i] % (cellsPerDimension*cellsPerDimension)) % (cellsPerDimension);
        int y = (keys[i] % (cellsPerDimension*cellsPerDimension)) / (cellsPerDimension);
        int z = (keys[i] / (cellsPerDimension*cellsPerDimension));
        vector<int> point = {x,y,z};
        results.push_back(point);        
    }

    
    ofstream output;
    char filename[23];
    sprintf(filename, "fullSearchOutput%d.txt", shift_i);
    output.open(filename,fstream::out | ios_base::trunc);
    for (int i = 0; i < (int) results.size(); ++i)
    {
        output<<results[i][0]<<' '<<results[i][1]<<' '<<results[i][2]<<endl;
    }
    output.close();
    
cout<<"The centers are stored to output.txt"<<endl;
}    
return 0;
}
