#include <iostream>
#include <cstdio>
#include <cstring>
#include <vector>
#include <cstring>
#include <cmath>
#include <fstream>

//#include <gadgetreader.hpp>
#include "simplereader.h"
#include "countSketch.h"


using namespace std;
//using namespace GadgetReader;


int main(int argc, const char* argv[]){
    if(argc != 5){
        printf("Usage: %s <FileNamePattern> <nFiles> <levelSize> <nCenters>\n", argv[0]);
        exit(1);
    }
    
    double levelSize = atof(argv[3]);
    int k = atoi(argv[4]);
    int nFiles = atoi(argv[2]);
    double error = 0.05;
    int b = 9999999;
    
    GadgetSimpleReader testreader;
    gadget_header head;
    
    string fileNamePattern = argv[1];
    head = testreader.getHeader(fileNamePattern + to_string(0), 1, true, false, true);
    
    double boxSize = head.BoxSize;
    cout<<"Boxsize is "<<boxSize<<endl;
    
    long ncells = (boxSize*boxSize*boxSize)/(levelSize*levelSize*levelSize);
    int t = log2(ncells/error);
    
    vector<vector<int> > shifts;
    shifts.push_back({0,0,0});
    shifts.push_back({0,0,1});
    shifts.push_back({0,1,0});
    shifts.push_back({0,1,1});
    shifts.push_back({1,0,0});
    shifts.push_back({1,0,1});
    shifts.push_back({1,1,0});
    shifts.push_back({1,1,1});

    for (int shift_i = 0; shift_i < (int) shifts.size(); ++shift_i)
    {
        CountSketch CS(levelSize, ncells, boxSize, b, t, k);
        vector<int> shift = shifts[shift_i];
        
        printf("Start Count Sketch # %d shift calculation...\n", shift_i);

        
    
    
        for (int i =0; i<nFiles; i++) {
        
        string filename = fileNamePattern + to_string(i);
        printf("########Executing the chunk #%d #########\n",i);
        //GSnap snap(argv[1]);
        /*
         #########The simplier reader begin ##########
         */
        head = testreader.getHeader(filename, 1, true, false, true);
        /*
         #########The simplier reader end ##########
         */
        
        //int nparts = snap.GetNpart(DM_TYPE);
        
        vector<Particle> parts;
        printf("Reading particles from the data from file ... \n");
        parts = testreader.getParticle(filename, 1, true, false, true);
        int nparts = parts.size();
        //double boxSize = snap.GetHeader().BoxSize;
        //int t = log2(ncells/error);
        printf("Params: (levelsize %f) (nparts %d) (ncells %ld) (boxsize %f) (b %d) (t %d) \n",levelSize,nparts,ncells,boxSize,b,t);
        
        
        for(int i = 0; i < nparts; i++){
            double x = parts[i].Pos[0] + shift[0]*0.5*levelSize;
            double y = parts[i].Pos[1] + shift[1]*0.5*levelSize;
            double z = parts[i].Pos[2] + shift[2]*0.5*levelSize;
            CS.alg(x, y, z);
            if(i % 1000 == 0){
                printf("Please wait... %2.2f%%         \r", (double)i * 100 / nparts);
                cout.flush();
            }
        }
        printf("\n");
        
        
        //printf("Reading particles from the data file... \n");
        //vector<float> temppos = snap.GetBlock("POS ", nparts, 0, ~(1 << DM_TYPE));
        
    }
        
        ofstream output;
        char filename[23];
        sprintf(filename, "CSOutput%d.txt", shift_i);
        output.open(filename,fstream::out | ios_base::trunc);
        
        vector<vector<int> >results = CS.getResult();
        
        for (int i = 0; i < (int)results.size(); ++i)
        {
            output<<results[i][0]<<' '<<results[i][1]<<' '<<results[i][2]<<' '<<results[i][3]<<endl;
        }
        output.close();
        
        cout<<"The centers are stored...."<<endl;
    
}
    //snap.GetBlocks("POS ",data_array, snap.GetNpart(DM_TYPE), 0, 0);
    
    //map<int, Cell> sampledCells = sampler.getSampledCells();
    
    
    //printf("%d/%d cell opened. %2.2f%% of the cells are opened\n",
    
    //    for(it = sampledCells.begin(); it != sampledCells.end(); ++it){
    //        printf("%f %f %f %d\n",
    //                it->second.x, it->second.y, it->second.z, 
    //                it->second.count);
    //    }
    
    
    return 0;
}
